package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.enums.LoginRuleType;
import cn.com.qmth.examcloud.core.basic.service.LoginRuleService;
import cn.com.qmth.examcloud.core.basic.service.VerifyCodeService;
import cn.com.qmth.examcloud.core.basic.service.bean.GeetestLoginInfo;
import cn.com.qmth.examcloud.starters.greetest.model.RegisterReq;
import cn.com.qmth.examcloud.starters.greetest.model.RegisterResp;
import cn.com.qmth.examcloud.starters.greetest.model.ValidateResp;
import cn.com.qmth.examcloud.starters.greetest.service.GeetestService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import cn.com.qmth.examcloud.web.support.StatusResponseX;
import cn.com.qmth.examcloud.web.support.WithoutStackTrace;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * 验证码相关接口
 */
@RestController
@Api(tags = "验证码登录相关接口")
public class VerifyCodeController extends ControllerSupport {

    @Autowired
    private LoginRuleService loginRuleService;

    @Autowired
    private VerifyCodeService verifyCodeService;

    @Autowired
    private GeetestService geetestService;

    @Naked
    @ApiOperation(value = "极验-验证初始化接口")
    @PostMapping(value = "/api/ecs_core/verifyCode/register")
    public RegisterResp register(@RequestBody RegisterReq req, HttpServletRequest request) {
        req.setIp_address(super.getIp(request));
        return geetestService.register(req);
    }

    @Naked
    @WithoutStackTrace
    @ApiOperation(value = "极验-验证码登录接口")
    @PostMapping(value = "/api/ecs_core/verifyCode/gt/login")
    public StatusResponseX<User> geetestLogin(@RequestBody GeetestLoginInfo info, HttpServletRequest request) {
        setAlwaysOKResponse();

        if (info.getRootOrgId() == null) {
            throw new StatusException("400", "顶级机构ID不能为空");
        }

        // 当前机构是否“开放极验验证码登录”
        boolean open = loginRuleService.isLoginRuleAllow(info.getRootOrgId(), LoginRuleType.GEETEST_LOGIN);
        if (open) {
            info.setIp_address(super.getIp(request));
            ValidateResp resp = geetestService.validate(info);
            if (!resp.getSuccess()) {
                throw new StatusException("400", resp.getMsg());
            }
        }

        User user = verifyCodeService.geetestLogin(info);
        return new StatusResponseX<>(user);
    }

    /*@Naked
    @Deprecated
    @ApiOperation(value = "数字公式-验证码生成接口（待废弃）")
    @PostMapping(value = "/api/ecs_core/verifyCode/generate")
    public String generate(@RequestParam Long rootOrgId, @RequestParam String accountValue) {
        return verifyCodeService.verifyCodeGenerate(rootOrgId, accountValue);
    }

    @Naked
    @Deprecated
    @WithoutStackTrace
    @ApiOperation(value = "数字公式-验证码登录接口（待废弃）")
    @PostMapping(value = "/api/ecs_core/verifyCode/login")
    public StatusResponseX<User> login(@RequestBody VerifyCodeLoginInfo info, HttpServletRequest request) {
        setAlwaysOKResponse();

        info.setClientIp(super.getIp(request));
        User user = verifyCodeService.verifyCodeLogin(info);
        return new StatusResponseX<>(user);
    }

    @Naked
    @ApiOperation(value = "Resource接口（待废弃）")
    @GetMapping(value = {"/resource.js"})
    public void resource(@RequestParam String u) {
        verifyCodeService.verifyCodeResource(u);
    }*/

}