package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.api.commons.enums.CourseLevel;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetCoursesByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetCoursesByIdListResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.CourseService;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseInfo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * {@link StatusException} 状态码范围：011XXX<br>
 *
 * @author WANGWEI
 * @date 2018年7月10日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-课程相关接口")
@RequestMapping("${$rmp.cloud.basic}" + "course")
public class CourseCloudServiceProvider implements CourseCloudService {

    private static final long serialVersionUID = -2733647860311223743L;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    CourseRepo courseRepo;

    @Autowired
    CourseService courseService;

    @ApiOperation(value = "保存课程")
    @PostMapping("saveCourse")
    @Transactional
    @Override
    public SaveCourseResp saveCourse(@RequestBody SaveCourseReq courseReq) {

        CourseInfo info = new CourseInfo();
        info.setRootOrgId(courseReq.getRootOrgId());
        info.setCode(courseReq.getCourseCode());
        info.setEnable(courseReq.getEnable());
        info.setId(courseReq.getCourseId());
        info.setName(courseReq.getCourseName());
        if (null != courseReq.getCourseLevel()) {
            info.setLevel(CourseLevel.getCourseLevel(courseReq.getCourseLevel()));
        }

        CourseEntity saved = courseService.saveCourse(info);

        SaveCourseResp resp = new SaveCourseResp();
        CourseBean courseBean = new CourseBean();
        courseBean.setId(saved.getId());
        courseBean.setCode(saved.getCode());
        courseBean.setLevel(saved.getLevel().name());
        courseBean.setName(saved.getName());
        courseBean.setRootOrgId(saved.getRootOrgId());
        courseBean.setEnable(saved.getEnable());

        resp.setCourseBean(courseBean);

        return resp;
    }

    @ApiOperation(value = "查询课程")
    @PostMapping("getCourse")
    @Override
    public GetCourseResp getCourse(@RequestBody GetCourseReq req) {
        GetCourseResp resp = new GetCourseResp();
        Long rootOrgId = req.getRootOrgId();
        String code = req.getCode();
        Long id = req.getId();

        if (null == rootOrgId) {
            throw new StatusException("160000", "rootOrgId is null");
        }
        OrgEntity rootOrg = GlobalHelper.getEntity(orgRepo, rootOrgId, OrgEntity.class);
        if (null == rootOrg) {
            throw new StatusException("160001", "机构不存在");
        }
        if (null != rootOrg.getParentId()) {
            throw new StatusException("160002", "机构错误");
        }

        CourseEntity c = null;
        if (null != id) {
            c = GlobalHelper.getEntity(courseRepo, id, CourseEntity.class);

        } else if (StringUtils.isNotBlank(code)) {
            c = courseRepo.findByRootOrgIdAndCode(rootOrgId, code);
        } else {
            throw new StatusException("160002", "id,code can not be all null");
        }

        if (null == c) {
            throw new StatusException("160003", "课程不存在");
        }

        CourseBean courseBean = new CourseBean();
        courseBean.setId(c.getId());
        courseBean.setCode(c.getCode());
        courseBean.setLevel(c.getLevel().name());
        courseBean.setName(c.getName());
        courseBean.setRootOrgId(c.getRootOrgId());
        courseBean.setEnable(c.getEnable());
        resp.setCourseBean(courseBean);

        return resp;
    }

    @ApiOperation(value = "按ID查询课程")
    @PostMapping("getCoursesByIdList")
    @Override
    public GetCoursesByIdListResp getCoursesByIdList(@RequestBody GetCoursesByIdListReq req) {
        List<Long> courseIdList = req.getCourseIdList();
        final List<CourseBean> list = Lists.newArrayList();
        for (Long cur : courseIdList) {
            CourseEntity c = GlobalHelper.getEntity(courseRepo, cur, CourseEntity.class);
            if (null == c) {
                throw new StatusException("160001", "no course [id=" + cur + "]");
            }
            CourseBean courseBean = new CourseBean();
            courseBean.setId(c.getId());
            courseBean.setCode(c.getCode());
            courseBean.setLevel(c.getLevel().name());
            courseBean.setName(c.getName());
            courseBean.setRootOrgId(c.getRootOrgId());
            courseBean.setEnable(c.getEnable());

            list.add(courseBean);
        }

        GetCoursesByIdListResp resp = new GetCoursesByIdListResp();
        resp.setCourseList(list);
        return resp;
    }

}
